use rusqlite::{Error, Result};
use serde::{Deserialize, Serialize};
use std::collections::HashMap;

#[derive(Debug, Serialize, Deserialize)]
pub struct Cat {
    id: i32,
    name: String,
    color: String,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Color {
    id: i32,
    name: String,
}

pub type CatsResult = Result<Vec<Cat>, Error>;
pub type ColorsResult = Result<Vec<Color>, Error>;
pub type CatResult = Result<Cat, Error>;
pub type ColorResult = Result<Color, Error>;
pub type Pool = r2d2::Pool<r2d2_sqlite::SqliteConnectionManager>;

// Сервис базы данных
#[derive(std::clone::Clone)]
pub struct Db {
    pool: Pool,
}

impl Db {
    pub fn new(pool: Pool) -> Db {
        return Db { pool: pool };
    }
    // Создание сервиса
    pub async fn init(&self) {
        let conn = match self.pool.get() {
            Ok(conn) => conn,
            Err(err) => panic!("{}", err),
        };

        conn.execute(
            "create table if not exists cat_colors (
                    id integer primary key,
                    name text not null unique
                )",
            [],
        )
        .ok();
        conn.execute(
            "create table if not exists cats (
                    id integer primary key,
                    name text not null,
                    color_id integer not null references cat_colors(id)
                )",
            [],
        )
        .ok();
        let mut cat_colors = HashMap::new();
        cat_colors.insert(String::from("Blue"), vec!["Tigger", "Sammy"]);
        cat_colors.insert(String::from("Black"), vec!["Oreo", "Biscuit"]);
        for (color, catnames) in &cat_colors {
            conn.execute(
                "INSERT INTO cat_colors (name) values (?1)",
                &[&color.to_string()],
            )
            .ok();
            let last_id: String = conn.last_insert_rowid().to_string();
            for cat in catnames {
                conn.execute(
                    "INSERT INTO cats (name, color_id) values (?1, ?2)",
                    &[&cat.to_string(), &last_id],
                )
                .ok();
            }
        }
    }

    // Получение кошек
    pub fn get_cats(&self) -> CatsResult {
        let conn = match self.pool.get() {
            Ok(conn) => conn,
            Err(err) => panic!("{}", err),
        };
        let mut stmt = conn.prepare(
            "SELECT c.id, c.name, cc.name from cats c INNER JOIN cat_colors cc ON cc.id = c.color_id;",
        )?;
        return stmt
            .query_map([], |row| {
                Ok(Cat {
                    id: row.get(0)?,
                    name: row.get(1)?,
                    color: row.get(2)?,
                })
            })
            .and_then(Iterator::collect);
    }
    // Получение  cлучайной кошки
    pub fn get_random_cat(&self) -> CatResult {
        let conn = match self.pool.get() {
            Ok(conn) => conn,
            Err(err) => panic!("{}", err),
        };
        return conn.query_row("SELECT c.id, c.name, cc.name from cats c INNER JOIN cat_colors cc ON cc.id = c.color_id ORDER BY random() LIMIT 1;",
            [],
            |row| {
                Ok(Cat {
                    id: row.get(0)?,
                    name: row.get(1)?,
                    color: row.get(2)?,
                })
            },
        );
    }

    // Получение цветов
    pub fn get_colors(&self) -> ColorsResult {
        let conn = match self.pool.get() {
            Ok(conn) => conn,
            Err(err) => panic!("{}", err),
        };
        let mut stmt = conn.prepare("SELECT id, name from cat_colors")?;
        return stmt
            .query_map([], |row| {
                Ok(Color {
                    id: row.get(0)?,
                    name: row.get(1)?,
                })
            })
            .and_then(Iterator::collect);
    }

    // Добавление кошки
    pub fn add_cat(&self, cat_name: String, cat_color: String) -> CatResult {
        let conn = match self.pool.get() {
            Ok(conn) => conn,
            Err(err) => panic!("{}", err),
        };
        return conn.query_row("INSERT INTO cats (name, color_id) SELECT ?1, id FROM cat_colors cc WHERE name = ?2 RETURNING id, name, ?2",
            [cat_name, cat_color],
            |row| {
                Ok(Cat {
                    id: row.get(0)?,
                    name: row.get(1)?,
                    color: row.get(2)?,
                })
            },
        );
    }
    // Добавление цвета
    pub fn add_color(&self, color_name: String) -> ColorResult {
        let conn = match self.pool.get() {
            Ok(conn) => conn,
            Err(err) => panic!("{}", err),
        };
        return conn.query_row(
            "INSERT INTO cat_colors (name) VALUES (?1) RETURNING id, name",
            [color_name],
            |row| {
                Ok(Color {
                    id: row.get(0)?,
                    name: row.get(1)?,
                })
            },
        );
    }
}
